
# q-expansion of newform 152.6.a.b, downloaded from the LMFDB on 08 February 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 152
weight = 6
poly_data = [1118, -267, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[-181, 1, 1], 7], [[-369, 30, 2], 7]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [-2, 1, 0], [20, 1, 1], [-67, -2, -2], [152, -25, 5], [-481, -21, -5], [-811, 18, -24], [361, 0, 0], [-805, -75, 67], [1428, 155, -98], [3559, 30, -45], [-4343, 38, 63], [-7357, 496, -127], [-2746, 129, -137], [-8966, -241, -27], [-25306, -483, 122], [13455, -845, 333], [7508, -1329, -171], [-30818, 679, 2], [2448, 2380, -774], [-6853, -1412, 514], [28613, 2772, 213], [21586, -1270, -858], [30369, -2454, 1293], [16844, 2866, 356], [35640, 1920, 418], [-3744, 6684, -1086], [117509, 1589, -33], [-37068, 6211, 1876], [-64091, 5698, -1113], [-67551, 2790, -2745], [-90500, -197, -597], [-238697, -8334, -262], [11974, -4063, -1741], [184672, 6019, -1541], [239078, -4168, -496], [132434, 4640, 772], [-246354, 2400, 10810], [37787, 13688, 8869], [-19472, 26080, 2242], [-106739, 20984, -8809], [-330700, -26040, -3662], [53115, 36338, 3912], [-300523, 11880, -6703], [-42172, -27018, 5394], [-212437, 11274, 16688], [-557821, 30691, -7763], [-61427, -36210, 15039], [-459553, -38735, -3509], [-211734, 23177, -1195], [-155434, -8273, 15507], [6183, -14906, -23752], [-200057, -45236, 11625], [-552764, -52363, 6335], [744138, -26036, -22502], [376652, -56705, 7995], [838477, 35048, 1677], [381785, -43839, -879], [124054, -59189, -5991], [-683512, -46574, -19598], [182490, 93603, 3975], [165123, 78377, -23161], [-861249, 95410, -16909], [617587, -56712, -2090], [-1126813, -28253, 12165], [812545, 3375, -35741], [739188, 81289, 36480], [-219688, -91816, 26424], [1124560, -80031, 18075], [273040, -206331, 25791], [-2597669, -54061, -15335], [791935, 19184, -14960], [-494484, -135492, 10004], [-1606337, -13755, 12347], [-86011, 80659, -31121], [1688495, 164220, -17173], [3191688, -88297, -14599], [-2176998, 44635, -9903], [-2845141, -53930, 19815], [-3784021, -112254, 5815], [-401966, -135536, -81954], [-1257942, -102123, -24382], [3264072, -57202, 45952], [-2294973, 205338, -20327], [-2210485, -20696, 41645], [341172, 190489, -74569], [-4267756, 168966, -70284], [-4102691, -31482, 14514], [2641390, -30329, -619], [-3054238, 25767, -17447], [5103724, -123583, 44087], [-1658892, -200976, 62736], [1386388, -205164, -54430], [-2587094, -307628, -18210], [-904668, -69405, -83377], [940757, 76461, -7999], [-6172000, 320742, -69504], [1392313, -336526, -76111], [-6960997, -97499, 13677], [2557188, -45409, 19565], [-4025784, -363384, -41788], [-1764678, 560995, -2779], [-9857, 278600, 62529], [3947644, 78128, -25804], [-3019032, -165200, -80868], [-4616483, 232412, 157954], [1743740, -334305, -97617], [3418456, 700016, -46046], [3682163, -898, 62303], [-8166880, -247042, 83926], [-9997874, 232110, -4802], [6343134, -77195, 159201], [2908208, 414181, -80081], [1670732, 523006, -101904], [-4159328, 35821, -149059], [-9891220, -165586, 35148], [-525774, -534161, 147423], [-4737949, -127014, 255176], [-2767872, 265243, 96119], [92001, 332773, 180327], [-1005183, -323215, 9513], [-553776, -478730, 29162], [-6744340, 191249, 23430], [-573386, -569308, -314686], [1905274, 911709, 101881], [1527286, -762742, 164752], [4067690, 484520, -332752], [10077089, -725746, 131298], [521209, -654620, 64844], [-6430214, 284558, 122960], [3444406, 489137, 287947], [-4897521, -1310870, -157361], [-2354258, 176908, -51874], [707022, 786247, -70665], [-4541345, 172450, 47534], [23413197, -125658, -67274], [-14941923, 284559, -159349], [15090137, -115955, -388507], [-7030746, 21601, -40270], [762467, -571786, -264612], [15183683, 784635, -140053], [-6798178, 684343, -288099], [14522151, -959540, 150664], [-8257844, 439279, -157200], [4559107, -542411, 232875], [17481234, -662074, -242848], [-8349752, -577988, -312930], [-15681778, -851554, 219868], [2247558, 546387, 416695], [-1930029, 125862, 475161], [18251869, -885781, 183291], [-8668964, -297787, 567247], [6344396, -398813, -317283], [36375286, -17688, 86980], [-7415364, 174873, 28100], [-3095328, 246794, -103978], [23392795, -962385, -185329], [5515091, -378849, 568221], [19214087, 439028, 337454], [-8920383, -263331, -248517], [9976060, 1226600, 237504], [25215795, 1521544, -65845], [21992584, 158568, -207984], [-20064172, 352544, 97560], [-22138397, 601434, -295469], [-24166866, -898742, -439134], [-2910743, 973888, 415603], [5718382, 967511, -63603], [23628641, 993122, 19141], [15616476, 1151043, -57885], [6782510, 1225979, -16400], [-15403930, -958335, -366674], [-29555796, 287056, -286796], [19305455, 317341, -453619], [-22496360, -1431380, 140892], [5266260, 1136751, 607553], [22706094, 1548140, -177438], [33382, -2420035, 323669], [39998388, 520980, 165884], [54252744, -280239, -266039], [-27303241, 662626, -536406], [7161821, -1587733, 136605], [-16183734, -179608, 1247604], [-4113414, -271102, -430732], [-31215967, 219010, 906794], [-11842768, -772147, 764959], [-23612259, -3477621, -194393], [-39422212, 723270, -464894], [17232563, 2769642, -168455], [-13925231, -1474907, -582283], [5211087, -275158, -721399], [-35685888, -729139, 653833], [-7282883, 612870, -13059], [-16506700, 2207789, -497514], [19125040, 1833984, -824112], [-20465150, -3448224, -798866], [41290365, -3815452, 1080728], [-1428074, -3541666, 319940], [33308921, 2952921, 153251], [-4662571, 68206, 712552], [23993198, 367110, -778342], [46716777, 436956, -360485], [37305096, 3227149, 309280], [-6950007, -1136332, -549541], [7639000, 1741990, 213996], [14778624, -1517605, -216869], [25820331, 1050696, 751826], [44116579, 1861613, -492613], [-29031207, 1784396, 716320], [65844595, 1515419, -109251], [-25618035, -1899941, -967853], [22090494, 2466326, 396268], [-13229262, -1971087, 134715], [20650457, -3101621, 1358469], [75360953, 190440, -1141181], [-44623341, 3609054, -928715], [9958797, -1763163, 601237], [-13826298, -2687074, 548274], [-9813779, -1802988, -1089879], [62448958, -3555061, 503359], [28277796, -3874552, 1132438], [66221951, 1223318, -450455], [-3775117, 3549486, -152439], [-10670481, 3135868, 449190], [-17389773, 2339229, -776395], [40038386, 460151, -940965], [27168912, 1507212, 1467910], [-59733, -4861780, 33723], [-31586934, -1096156, 464012], [38413932, -1239445, 975881], [-9798190, 6478255, -1151177], [20123778, 7093343, -82890], [27236367, -6021306, 1245843], [-55506940, -3378900, 192658], [-20780840, -2392139, -1560833], [-42263053, -829912, 530640], [-65328848, -270063, 295607], [26941314, 999793, -1467481], [83301390, 2345310, -366330], [102533, -988802, 1378777], [-42991678, -1429482, 1861224], [25636898, -698719, 27737], [-40553487, 2943087, 56209], [-8636501, -2857727, 2102877], [25287215, 4993268, 1427877], [-2967864, -3884225, 297467], [-20722012, -4999797, -1662537], [-19973611, 744410, 883917], [-62223220, 2632172, -616916], [-50139681, 806412, 1413162], [9542050, -5431492, -823088], [-14492119, 1231668, -603466], [-44959183, 5559049, -17379], [39784907, -2836824, -1537421], [27090240, 6895901, -1067383], [78600964, 2983366, 225672], [-179139434, 21785, -82903], [-12489713, -3916556, 984637], [14706417, -3860762, -2345941], [-50395732, 1370920, -1941070], [-549195, 4325500, -74837], [69268610, -5572538, 380008], [-1406646, -4127951, 1399319], [70899255, -3513999, -614907], [30358934, -4207093, 174039], [-77598784, -8606821, -64336], [60665716, 9515199, -293600], [-73228003, 4575168, -501098], [-160961489, -683719, -932867], [146286792, 445571, -126683], [-25794158, 10313217, -1750702], [43552418, 287283, -465692], [102408335, 9104060, 777870], [-64835051, -3760908, 175062], [19369797, 7702264, -1574051], [20654293, 2372011, -4009803], [32292178, -3489809, 1614119], [-48133424, -2039711, 2826548], [58601846, -7332906, 2586424], [-27111452, -8822123, 183957], [64159650, -244616, -4067080], [-2200603, 4716137, -1524255], [-88623399, 249904, 1701816], [-25986023, 1948257, 1541067], [-67417392, -7686575, 867313], [-119495738, 971689, 843307], [139539425, 2387826, -2382982], [95356216, -7058986, -614340], [30623871, 1058440, -2713520], [161192190, 1153178, -2238362], [-5790820, 1936148, -531082], [18557968, 6543497, 1651045], [3655334, 1826830, 258190], [238194392, -2678686, -529674], [-159665096, 4486784, -433218], [65828504, -121394, -4244330], [-73611326, -2301466, 1106602], [195274648, -5585193, 1604977], [-120800594, -7862971, -591010], [56308802, 3596821, 2996821], [-47600106, -9367795, 819117], [205596277, 5757095, 282743], [-111356939, 9133886, -147170], [-13997868, -5279129, 1274540], [31562692, -8164961, 3575919], [-21466909, 4498096, 1552439], [-67116099, 3074293, 2825297], [34269488, 7965753, 1044754], [-181068234, 5633385, -2257333], [40272828, -9959738, 4819922], [-150853408, 2669429, 350117], [-40351526, 3522336, 1047624], [-35667976, -2299978, 4254344], [93742745, -2925228, -3029653], [27693802, 3241076, -709510], [16976967, -8744382, -1344609], [-63398666, 2811085, 1458889], [-16502059, 1670808, -114115], [34986113, 9990280, -1724542], [-58574604, 21085, -2402257], [55369992, -2822885, 2821690], [-109857525, -6310586, 4499122], [78293005, 12406861, 3202579], [-1809029, -9732240, 726003], [-190687561, 6273465, -2958135], [167162251, -397564, -2764057], [-139921233, 3176313, -1139789], [-48659772, 1556347, -929054], [-73611041, 7875680, 422813], [151499954, -2295177, 2326617], [-72550754, 15691425, -2041657], [41865281, -11296997, 4953263], [194793045, -4978587, 3720221], [38332068, -11670601, -2711711], [-108158686, -8137173, -1237345], [117782948, 4258207, -1476073], [-123157568, -15804421, -2374564], [342619903, 4800612, -865235], [14878725, 5574400, -4773782], [-2850857, 355279, -1876945], [-129986186, -11206021, 1171750], [-53436923, -7502714, 433232], [171585926, 776823, -183504], [280969427, 1265144, -1727077], [-46048457, 2909297, -2033399], [142460874, 12180928, 105506], [155291336, -2586184, 323190], [42163776, -4065825, -1303530], [-128614210, 18331118, -2841706], [147822228, -7738025, -1238293], [-53084753, 244336, -4758259], [146248570, 4864058, -3350626], [121114244, 17783053, -1668423], [12398656, 16357814, -5157646], [53268253, 192320, -8478339], [-33132110, -20793276, 914010], [-184964189, -9300871, 3726971], [11758720, 9936200, 6829052], [147547991, 8812110, 2778326], [130871247, 22144002, 4520889], [-32752184, -16564641, -2337025], [105224666, -5377692, 1840320], [-75523571, -24665074, -166153], [-208552254, 5817031, 2668466], [178662594, 8805001, -286439], [114563977, 9404200, 2756671], [-40243754, -21130744, -1483452], [57677862, 3568620, 3653980], [43479551, -23510648, 1246839], [-169673970, -15441406, 3320554], [-53205182, 13403283, 1899992], [158513301, -18860524, 5022852], [-119053350, -12749973, 5326032], [213720266, 164125, 839463], [2752460, -3706913, -4599429], [89079227, -12327817, -4824185], [-148687438, 6238948, -2868518], [101057592, 2724007, 2990917], [-239180311, -2494526, 5268590], [60554938, -18726128, 444482], [61049434, -10530924, 1793368], [-90292888, -21607285, 4297285], [-153958221, -8390722, 6314990], [-77693692, -9206257, 4325505], [57518870, -14431959, -269620], [-105782306, 19702518, -4154578], [-231499121, -4161677, 3427927], [81661188, -4464858, 9254122], [36340148, -33296587, 2652539], [-42786741, 9830548, -7939299], [-86770440, 3574916, -5488458], [69003963, 16682991, -4492375], [-5955878, 453410, 8883386], [-365529982, -12750283, 2381273], [-171651914, 7963734, -2686604], [182801076, -21020204, -618374], [-411137615, 21735392, -5092683], [-40712805, 33060656, -388451], [220450629, 14583664, 3092523], [-361132862, -2838454, 987712], [-116287605, 15238588, 8828511], [-216069248, 5129359, -4259199], [309044506, 1753006, 103922], [-37941212, 10092086, -6692510], [210872253, -13002508, 5926917], [263011407, 27690568, 1771162], [-324416892, 24896069, -7258281], [-438977425, 5490300, -1330587], [-103902428, 7655703, -7597210], [-387358852, -1980121, 2145735], [187937293, 1135306, 166621], [-343397216, 12713827, 4487277], [-473175043, -3965693, -217229], [156932182, 5584833, -14402823], [-312196063, 10168507, -6727353], [227819270, -19744058, 6075520], [-145305968, 839226, 248944], [52626951, -14340170, 2941611], [-78739569, -19317727, -4817211], [51176652, 20077590, -346742], [93150507, -6934300, 3849387], [-270707064, -8605071, -728330], [-23910839, 16899782, -4327512], [138504232, -20204721, 8167782], [129706446, 1246846, 11570898], [247806427, 1540865, 2043611], [-3197422, -26169387, -1611060], [59843763, 29314072, 609008], [101162376, 13551261, -7831195], [-615686419, 3280948, 766800]]
